import { h, e as Host } from './p-c3592601.js';

/**
 * Computes selectors and returns them based on their priority.
 */
const computeSelectors = ({ element, size, states = {}, config = {}, }) => {
    let selectors = [];
    const data = config === null || config === void 0 ? void 0 : config.root[element];
    const add = (selector) => {
        selectors.push(selector);
        if (typeof size === 'string') {
            selectors.push(`${selector}.${size}`);
        }
    };
    add(element);
    if (typeof data === 'object' && !Array.isArray(data) && data !== null) {
        // check if the data variable is an object, strictly and not an array or just null
        const { state, states: elementStates } = data;
        let selector = element;
        let activeStates = [];
        if (Array.isArray(elementStates)) {
            activeStates = elementStates.filter((state) => states[state]);
            activeStates.sort();
            for (const state of activeStates) {
                add(`${selector}.${state}`);
            }
            if (activeStates.length > 1) {
                const booleanStateSelector = [selector, ...activeStates].join('.');
                add(booleanStateSelector);
            }
        }
        if (typeof state === 'string') {
            // rtk-meeting[meeting=joined]
            const keyValueSelector = `${element}[${state}=${states[state]}]`;
            add(keyValueSelector);
            for (const state of activeStates) {
                add(`${keyValueSelector}.${state}`);
            }
            if (activeStates.length > 1) {
                const withBooleanStateSelector = [keyValueSelector, ...activeStates].join('.');
                add(withBooleanStateSelector);
            }
        }
    }
    return selectors;
};
/**
 * Returns the computed styles - styles obtained from combining styles from all computed selectors
 * on the basis of their priorities.
 */
const getComputedStyles = ({ selectors, styles }) => {
    if (!Array.isArray(selectors) || styles == null)
        return {};
    const computedStyles = {};
    for (const selector of selectors) {
        const style = styles[selector];
        if (style != null) {
            Object.assign(computedStyles, style);
        }
    }
    return computedStyles;
};
/**
 * Returns the computed children which are to be rendered inside an element
 */
const getComputedChildren = ({ selectors, root }) => {
    if (!root || !Array.isArray(selectors))
        return [];
    let children = [];
    for (const selector of selectors) {
        const el = root[selector];
        if (Array.isArray(el)) {
            children = [...el];
        }
        else if (el) {
            if (el.children) {
                children = [...el.children];
            }
            if (Array.isArray(el.remove)) {
                for (const toRemove of el.remove) {
                    children = children.filter((child) => {
                        if (typeof child === 'string') {
                            return child !== toRemove;
                        }
                        else if (Array.isArray(child)) {
                            return child[0] !== toRemove;
                        }
                        return true;
                    });
                }
            }
            if (el.addBefore) {
                for (const [beforeEl, toAdd] of Object.entries(el.addBefore)) {
                    const idx = children.findIndex((child) => {
                        if (typeof child === 'string') {
                            return child === beforeEl;
                        }
                        else if (Array.isArray(child)) {
                            return child[0] === beforeEl;
                        }
                        return false;
                    });
                    if (idx >= 0) {
                        children.splice(idx, 0, ...toAdd);
                    }
                }
            }
            if (Array.isArray(el.add)) {
                children = children.concat(el.add);
            }
            if (Array.isArray(el.prepend)) {
                children = el.prepend.concat(children);
            }
        }
    }
    return children;
};

/**
 * Renders the children of an element.
 */
const RenderChildren = ({ elements, defaults, props = {}, deepProps = false, elementProps = {}, }) => {
    if (!Array.isArray(elements) || elements.length === 0)
        return null;
    return elements.map((element) => {
        return (h(Render, { element: element, defaults: defaults, props: props, childProps: deepProps && props, elementProps: elementProps }));
    });
};
const lenChildren = ({ element, defaults, props = {}, elementProps = {} }) => {
    var _a;
    const { config, size, states } = defaults;
    let Tag, configProps = {}, configChildren = [];
    if (Array.isArray(element)) {
        // get props if element is passed in array form:
        // ['rtk-participant-tile', { variant: 'gradient' }]
        [Tag, configProps, ...configChildren] = element;
    }
    else {
        Tag = element;
    }
    const elemData = (_a = config === null || config === void 0 ? void 0 : config.root) === null || _a === void 0 ? void 0 : _a[Tag];
    if (elemData != null && 'props' in elemData) {
        props = Object.assign(Object.assign({}, elemData['props']), props);
    }
    props = Object.assign(Object.assign({}, props), configProps);
    if (Tag in elementProps) {
        props = Object.assign(Object.assign({}, props), elementProps[Tag]);
    }
    const selectors = computeSelectors({ element: Tag, size, states, config });
    const computedChildren = getComputedChildren({ selectors, root: config.root });
    return computedChildren.length;
};
/**
 * Renders an element from UI Config
 */
const Render = ({ element, defaults, childProps = {}, props = {}, onlyChildren = false, asHost = false, deepProps = false, elementProps = {}, }, children) => {
    var _a;
    const { config, size, states } = defaults;
    let Tag, configProps = {}, configChildren = [];
    if (Array.isArray(element)) {
        // get props if element is passed in array form:
        // ['rtk-participant-tile', { variant: 'gradient' }]
        [Tag, configProps, ...configChildren] = element;
    }
    else {
        Tag = element;
    }
    const elemData = (_a = config === null || config === void 0 ? void 0 : config.root) === null || _a === void 0 ? void 0 : _a[Tag];
    if (elemData != null && 'props' in elemData) {
        props = Object.assign(Object.assign({}, elemData['props']), props);
    }
    props = Object.assign(Object.assign({}, props), configProps);
    if (Tag in elementProps) {
        props = Object.assign(Object.assign({}, props), elementProps[Tag]);
    }
    const selectors = computeSelectors({ element: Tag, size, states, config });
    const computedChildren = getComputedChildren({ selectors, root: config.root });
    if (onlyChildren) {
        return (h(RenderChildren, { elements: computedChildren, defaults: defaults, props: childProps, deepProps: deepProps, elementProps: elementProps }));
    }
    const styles = getComputedStyles({ selectors, styles: config.styles });
    if (asHost) {
        return (h(Host, Object.assign({}, defaults, { style: styles }, props),
            h(RenderChildren, { elements: computedChildren, defaults: defaults, props: childProps, deepProps: deepProps, elementProps: elementProps }),
            children,
            configChildren.map((child) => {
                if (Array.isArray(child)) {
                    const [Tag, props] = child;
                    return h(Tag, Object.assign({}, defaults, props));
                }
                return child;
            })));
    }
    if (['rtk-header', 'rtk-controlbar'].includes(Tag)) {
        props['disableRender'] = true;
    }
    if (Tag.startsWith('rtk-')) {
        return (h(Tag, Object.assign({}, defaults, { style: styles }, props),
            h(RenderChildren, { elements: computedChildren, defaults: defaults, props: childProps, deepProps: deepProps, elementProps: elementProps }),
            children,
            configChildren.map((child) => {
                if (Array.isArray(child)) {
                    const [Tag, props] = child;
                    return h(Tag, Object.assign({}, defaults, props));
                }
                return child;
            })));
    }
    else {
        const [HTMLTag, id] = Tag.split('#');
        return (h(HTMLTag, Object.assign({ id: id, style: styles }, props),
            h(RenderChildren, { elements: computedChildren, defaults: defaults, props: childProps, deepProps: deepProps, elementProps: elementProps }),
            children,
            configChildren.map((child) => {
                if (Array.isArray(child)) {
                    const [Tag, props] = child;
                    return h(Tag, Object.assign({}, defaults, props));
                }
                return child;
            })));
    }
};

export { Render as R, lenChildren as l };
